#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2019 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Encodings.RealVectorEncoding;
using HeuristicLab.Operators;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Problems.MovingPeaksBenchmark {
  [Item("MovingPeaksBenchmarkProblemEvaluator", "Evaluation operator for the Moving Peaks Benchmark.")]
  [StorableClass]
  public class MovingPeaksBenchmarkProblemEvaluator : InstrumentedOperator, IMovingPeaksBenchmarkProblemEvaluator {
    public ILookupParameter<DoubleMatrix> PeakLocationsParameter {
      get { return (ILookupParameter<DoubleMatrix>)Parameters["PeakLocations"]; }
    }
    public ILookupParameter<DoubleArray> PeakWidthsParameter {
      get { return (ILookupParameter<DoubleArray>)Parameters["PeakWidths"]; }
    }
    public ILookupParameter<DoubleArray> PeakHeightsParameter {
      get { return (ILookupParameter<DoubleArray>)Parameters["PeakHeights"]; }
    }
    public ILookupParameter<RealVector> PointParameter {
      get { return (ILookupParameter<RealVector>)Parameters["Point"]; }
    }
    public ILookupParameter<DoubleValue> QualityParameter {
      get { return (ILookupParameter<DoubleValue>)Parameters["Quality"]; }
    }

    [StorableConstructor]
    protected MovingPeaksBenchmarkProblemEvaluator(bool deserializing) : base(deserializing) { }
    protected MovingPeaksBenchmarkProblemEvaluator(MovingPeaksBenchmarkProblemEvaluator original, Cloner cloner) : base(original, cloner) { }
    public MovingPeaksBenchmarkProblemEvaluator() : base() {
      Parameters.Add(new LookupParameter<DoubleMatrix>("PeakLocations", "Current position of the peaks."));
      Parameters.Add(new LookupParameter<DoubleArray>("PeakWidths", "Current width of the peaks."));
      Parameters.Add(new LookupParameter<DoubleArray>("PeakHeights", "Current height of the peaks."));
      Parameters.Add(new LookupParameter<RealVector>("Point", "The point which should be evaluated."));
      Parameters.Add(new LookupParameter<DoubleValue>("Quality", "Quality value of the evaluated point."));
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new MovingPeaksBenchmarkProblemEvaluator(this, cloner);
    }

    public override IOperation InstrumentedApply() {
      DoubleMatrix peaks = PeakLocationsParameter.ActualValue;
      DoubleArray widths = PeakWidthsParameter.ActualValue;
      DoubleArray heights = PeakHeightsParameter.ActualValue;
      RealVector point = PointParameter.ActualValue;
      double quality = Apply(peaks, widths, heights, point);
      QualityParameter.ActualValue = new DoubleValue(quality);
      return base.InstrumentedApply();
    }

    public double Apply(DoubleMatrix peaks, DoubleArray widths, DoubleArray heights, RealVector point) {
      return 0;
    }
  }
}
